"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserModel = void 0;
const StoreModel_1 = require("./StoreModel");
const auth_1 = require("firebase/auth");
class UserModel extends StoreModel_1.StoreModel {
    constructor(auth, firestoreDB, table) {
        super(table !== null && table !== void 0 ? table : "Users", firestoreDB);
        this._auth = auth;
    }
    /**
     * Create a new firebase user and sore the value
     * of data in firestore database
     * @param { string; string; object; } {email; password; userData;}
     * @returns
     */
    registerWithEmailAndPassword(_a) {
        return __awaiter(this, arguments, void 0, function* ({ email, password, userData }) {
            var _b;
            try {
                const user = yield ((_b = this._auth) === null || _b === void 0 ? void 0 : _b.createUserWithEmailAndPassword(email, password));
                // send verification mail
                if (user && userData) {
                    this.save(userData, user.user.uid);
                }
                return {
                    data: user,
                    status: 'success',
                    message: 'User registered successfully'
                };
            }
            catch (error) {
                if (error.code === 'auth/email-already-in-use') {
                    return {
                        status: 'error',
                        message: `${email} is already in use!`
                    };
                }
                if (error.code === 'auth/invalid-email') {
                    return {
                        status: 'error',
                        message: `${email} is an invalid email address`
                    };
                }
                return {
                    status: 'error',
                    message: 'Unable to register user'
                };
            }
        });
    }
    /**
     * Login user with email and password
     * @param param0
     * @returns
     */
    login(_a) {
        return __awaiter(this, arguments, void 0, function* ({ email, password, verifyEmail = false }) {
            try {
                const signedIn = yield this._auth.signInWithEmailAndPassword(email, password);
                // check if email should be verified
                if (verifyEmail) {
                    if (!signedIn.user.emailVerified) {
                        return {
                            status: 'error',
                            message: 'Email address needs verification'
                        };
                    }
                }
                return {
                    status: 'success',
                    message: 'User logged in successfully',
                    data: signedIn
                };
            }
            catch (error) {
                return {
                    status: 'error',
                    message: 'Invalid login credentials'
                };
            }
        });
    }
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this._auth.signOut();
                return {
                    status: 'success',
                    message: "User signout!"
                };
            }
            catch (_) {
                return { status: 'error', message: 'Unable to complete process' };
            }
        });
    }
    deleteAccount(user) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield Promise.all([
                    (0, auth_1.deleteUser)(user),
                    this.delete(user.uid)
                ]);
                return { status: 'success', message: 'Account deleted successfully' };
            }
            catch (error) {
                return { status: 'error', message: "unable to delete user account" };
            }
        });
    }
}
exports.UserModel = UserModel;
