"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StoreModel = void 0;
const firestore_1 = __importStar(require("@react-native-firebase/firestore"));
class StoreModel {
    _table;
    _firestoreDB;
    constructor(table, firestoreDB) {
        this._firestoreDB = firestoreDB;
        this._table = this._firestoreDB.collection(table);
    }
    /**
     *
     * @param { string } id
     * @returns
     */
    async find(id) {
        try {
            const docSnap = await this._table.doc(id).get();
            if (docSnap.exists) {
                return {
                    data: { ...docSnap.data(), reference: id },
                    message: "Document found",
                    status: 'success'
                };
            }
            return { message: 'Document does not exist', status: 'success' };
        }
        catch (_) {
            return { message: 'Document does not exist', status: 'error' };
        }
    }
    /**
     * Fetch all data in a document or range of data
     * by the document references
     * @param ids
     * @returns
     */
    async findAll(ids) {
        try {
            if (ids) {
                const results = [];
                for (let id of ids) {
                    const item = await this.find(id);
                    if (item.data) {
                        results.push(item);
                    }
                }
                return { data: results, message: 'Documents found', status: 'success' };
            }
            else {
                const snapshots = await this._table.get();
                if (!snapshots.empty) {
                    const data = snapshots.docs.map((document) => {
                        return { ...document.data(), reference: document.id };
                    });
                    return { data, message: 'Documents found', status: 'success' };
                }
                else {
                    return { message: 'Documents empty', status: 'success' };
                }
            }
        }
        catch (_) {
            return { message: 'Document does not exist', status: 'error' };
        }
    }
    async findWhereOrAnd({ wh, lim, order, offset }) {
        try {
            // set where clause
            const andWhere = [];
            const orWhere = [];
            let queryRef = this._table;
            // add where parameter
            if (wh) {
                wh.parameter.forEach((clause) => {
                    const { key, operator, value } = clause;
                    const whe = (0, firestore_1.Filter)(key, operator, value);
                    clause.type === 'and' ?
                        andWhere.push(whe) : orWhere.push(whe);
                });
                if (wh.type === 'andOr') {
                    queryRef = this._table.where(firestore_1.Filter.and(...andWhere, firestore_1.Filter.or(...orWhere)));
                }
                else if (wh.type === 'or') {
                    queryRef = firestore_1.Filter.or(...orWhere);
                }
                else {
                    queryRef = firestore_1.Filter.and(...andWhere);
                }
            }
            // let constraint = []
            // add order by
            if (order) {
                queryRef.orderBy(order);
            }
            // add offset
            if (offset) {
                const off = await this._table.doc(offset).get();
                queryRef.startAfter(off.data());
            }
            // add limit
            if (lim) {
                queryRef.limit(lim);
            }
            // fetch data
            const snapshot = await this._table.get();
            if (!snapshot.empty) {
                const data = snapshot.docs.map(document => {
                    return { ...document.data(), reference: document.id };
                });
                return { data, message: 'Documents found', status: 'success' };
            }
            else {
                return { message: 'No Documents found', status: 'success' };
            }
        }
        catch (_) {
            return { message: 'No Documents found', status: 'error' };
        }
    }
    async findWhere({ wh, lim, order, offset }) {
        try {
            // set where clause
            const andWhere = [];
            wh ? wh.forEach((clause) => {
                const { key, operator, value } = clause;
                const whe = (0, firestore_1.Filter)(key, operator, value);
                andWhere.push(whe);
            }) : [];
            // let constraint: QueryConstraint[] = []
            // add where parameter
            if (wh) {
                this._table.where(firestore_1.Filter.and(...andWhere));
            }
            // add order by
            if (order) {
                this._table.orderBy(order);
            }
            // add offset
            if (offset) {
                const off = await this._table.doc(offset).get();
                this._table.startAfter(off.data());
            }
            // add limit
            if (lim) {
                this._table.limit(lim);
            }
            const snapshot = await this._table.get();
            if (!snapshot.empty) {
                const data = snapshot.docs.map(document => {
                    return { ...document.data(), reference: document.id };
                });
                return { data, message: 'Documents found', status: 'success' };
            }
            else {
                return { message: 'No Documents found', status: 'success' };
            }
        }
        catch (error) {
            return { message: 'No Documents found', status: 'error' };
        }
    }
    async save(data, id) {
        delete data.reference;
        try {
            if (id === undefined) {
                const documentRef = await this._table.add(data);
                return { status: 'success', message: 'Document added successfully!', data: documentRef.id };
            }
            else {
                // await setDoc(doc(this.firestorDB!, this.table, id!), data)
                await this._table.doc(id).set(data);
                return { status: 'success', message: 'Document updated successfully!', data: id };
            }
        }
        catch (_) {
            return { status: 'error', message: 'Unable to save document' };
        }
    }
    async delete(id) {
        try {
            await this._table.doc(id).delete();
            return { status: 'success', message: 'Document deleted successfully!' };
        }
        catch (error) {
            return { status: 'success', message: 'Unable to delete document' };
        }
    }
    async update(data, id) {
        try {
            delete data.reference;
            const result = await this._table.doc(id).update(data);
            return { data: result, message: 'Data updated successfully', status: 'success' };
        }
        catch (error) {
            return { message: 'Unable to update document', status: 'error' };
        }
    }
    stream(callBack, id) {
        try {
            if (id) {
                this._table.doc(id).onSnapshot((doc) => {
                    callBack({ ...doc.data(), reference: id });
                });
            }
            else {
                this._table.onSnapshot((coll) => {
                    callBack(coll.docs.map((value) => {
                        const data = { ...value.data(), reference: value.id };
                        return data;
                    }));
                });
            }
        }
        catch (_) {
            return;
        }
    }
    async streamWhere(wh, callBack, lim, order, offset) {
        try {
            const andWhere = [];
            wh ? wh.forEach((clause) => {
                const { key, operator, value } = clause;
                const whe = (0, firestore_1.Filter)(key, operator, value);
                andWhere.push(whe);
            }) : [];
            // add where parameter
            if (wh) {
                this._table.where(firestore_1.Filter.and(...andWhere));
            }
            // add order by
            if (order) {
                this._table.orderBy(order);
            }
            // add offset
            if (offset) {
                const off = await this._table.doc(offset).get();
                this._table.startAfter(off.data());
            }
            // add limit
            if (lim) {
                this._table.limit(lim);
            }
            this._table.onSnapshot((snapShot) => {
                callBack(snapShot.docs.map((value) => {
                    const data = { ...value.data(), reference: value.id };
                    return data;
                }));
            });
        }
        catch (error) {
            return;
        }
    }
    async countData(where) {
        try {
            // set parameter
            const andWhere = [];
            where ? where.forEach((clause) => {
                const { key, operator, value } = clause;
                const whe = (0, firestore_1.Filter)(key, operator, value);
                andWhere.push(whe);
            }) : [];
            this._table.where(firestore_1.Filter.and(...andWhere));
            const data = await this._table.countFromServer().get();
            return { message: "Counted data", status: 'success', data: data.data() };
        }
        catch (_) {
            return { message: "Unable to cound data", status: "error" };
        }
    }
    saveBatch({ data }) {
        throw new Error("Method not implemented.");
    }
    updateBatch({ data }) {
        throw new Error("Method not implemented.");
    }
    deleteBatch({ ids }) {
        throw new Error("Method not implemented.");
    }
    async incrementDecrement({ dbReference, key, isIncrement, incrementalValue }) {
        try {
            const docRef = this._table.doc(dbReference);
            const value = isIncrement ? incrementalValue ?? 1 : (incrementalValue ?? 1) * -1;
            await docRef.update({ [key]: firestore_1.default.FieldValue.increment(value) });
            return { status: 'success', message: "Counter successful" };
        }
        catch (error) {
            return { status: 'error', message: "unable to count data" };
        }
    }
}
exports.StoreModel = StoreModel;
